@@ -1,7 +1,7 @@
 #
 # Reportbug module - common functions for reportbug and greportbug
 #   Written by Chris Lawrence <lawrencc@debian.org>
-#   (C) 1999-2000 Chris Lawrence
+#   (C) 1999-2002 Chris Lawrence
 #
 # This program is freely distributable per the following license:
 #
@@ -21,82 +21,15 @@
 #
 # Version ##VERSION##; see changelog for revision history
 
-import time, sys, string, os, locale, re, pwd, commands, shlex
+import time, sys, string, os, locale, re, pwd, commands, shlex, debianbts
+import rfc822, socket
 
-# Severity levels
-SEVERITIES = {'c':'critical',
-              'g':'grave',
-              's':'serious',
-              'i':'important',
-              'n':'normal',
-              'm':'minor',
-              'f':'fixed',
-              'w':'wishlist'}
+# Headers other than these become email headers for debbugs servers
+PSEUDOHEADERS = ('Package', 'Version', 'Severity', 'File', 'Tags',
+                 'Justification', 'Followup-For')
 
-# Rank order of severities, for sorting
-SEVLIST = ['critical', 'grave', 'serious', 'important', 'normal', 'minor',
-           'wishlist', 'fixed']
-
-# These packages are virtual in Debian; we don't look them up...
-debother = {
-    'base' : 'General bugs in the base system',
-    'boot-floppies' : 'Bugs in the boot and root disks',
-    'bugs.debian.org' : 'The bug tracking system, @bugs.debian.org',
-    'ftp.debian.org' : 'Problems with the main FTP site (or mirrors)',
-    'nonus.debian.org' : 'Problems with the non-us FTP site (or mirrors)',
-    'www.debian.org' : 'Problems with the website (or mirrors)',
-    'manual' : 'Bugs in the manual',
-    'project' : 'Problems related to Project administration',
-    'general' : 'Widespread problems (e.g., that many manpages are mode 755)',
-    'kernel' : 'Problems with the kernel in general (otherwise: kernel-image)',
-    'lists.debian.org' : 'The mailing lists (debian-*@lists.debian.org)',
-    'listarchives' : 'The mailing list archives.' }
-
-# Supported servers
-# Theoretically support for GNATS and Jitterbug could be added here.
-
-SYSTEMS = { 'debian' :
-            { 'name' : 'Debian', 'email': '%s@bugs.debian.org',
-              'btsroot' : 'http://www.debian.org/Bugs/',
-              'query-dpkg' : 1, 'type' : 'debbugs',
-              'otherpkgs' : debother,
-              'ldap' : ('bugs.debian.org', 35567,
-                        'ou=Bugs,o=Debian Project,c=US'),
-              'mirrors' :
-              { 'us' : 'http://www.debian.org/Bugs/',
-                'uk' : 'http://www.chiark.greenend.org.uk/debian/Bugs/',
-                'nl' : 'http://www.nl.debian.org/Bugs/',
-                'de' : 'http://www.infodrom.north.de/Debian/Bugs/',
-                }
-              },
-            'tdyc' :
-            { 'name' : 'TDYC [Debian KDE]',
-              'email': '%s@bugs.tdyc.com',
-              'btsroot' : 'http://bugs.tdyc.com/',
-              'otherpkgs' : {}, 'type' : 'debbugs',
-              'namefmt' : '%s-debian', 'query-dpkg' : 1,
-              'mirrors' : {} },
-            'kde' :
-            { 'name' : 'KDE Project', 'email': '%s@bugs.kde.org',
-              'btsroot': 'http://bugs.kde.org/', 'type' : 'debbugs',
-              'query-dpkg' : 0, 'otherpkgs' : {},
-              'mirrors' : {} },
-            'mandrake' :
-            { 'name' : 'Mandrake Linux', 'email': '%s@bugs.linux-mandrake.com',
-              'btsroot': 'http://www.linux-mandrake.com/bugs/',
-              'type' : 'debbugs', 'query-dpkg' : 0, 'otherpkgs' : {},
-              'mirrors' : {} },
-            'gnome' :
-            { 'name' : 'GNOME Project', 'email': '%s@bugs.gnome.org',
-              'btsroot': 'http://bugs.gnome.org/',
-              'type' : 'debbugs', 'mirrors' : {},
-              'query-dpkg' : 0, 'otherpkgs' : {} },
-            }
-
-# Convenient aliases
-SYSTEMS['kde-debian'] = SYSTEMS['tdyc']
-
-PSEUDOHEADERS = ('Package', 'Version', 'Severity', 'File')
+#VALID_UIS = ('newt', 'text')
+VALID_UIS = ('text',)
 
 def rfcdatestr(timeval = None):
     """Cheesy implementation of an RFC 822 date."""
@@ -114,9 +47,12 @@ def rfcdatestr(timeval = None):
         if mins and hrs < 0:
             hrs = hrs + 1
 
+        current = locale.setlocale(locale.LC_TIME)
         locale.setlocale(locale.LC_TIME, "C")
-        return time.strftime('%a, %d %b %Y %H:%M:%S', tm) + (" %+03d%02d" %
-                                                             (hrs, mins))
+        ret = time.strftime('%a, %d %b %Y %H:%M:%S', tm) + (" %+03d%02d" %
+                                                            (hrs, mins))
+        locale.setlocale(locale.LC_TIME, current)
+        return ret
     except:
         return 'Invalid date: '+`timeval`
 
@@ -141,7 +77,8 @@ def realpath(filename):
 
     return filename
 
-pathdirs = ['/usr/sbin', '/usr/bin', '/sbin', 'bin']
+pathdirs = ['/usr/sbin', '/usr/bin', '/sbin', '/bin', '/usr/X11R6/bin',
+            '/usr/games']
 
 def search_path_for(filename):
     dir, file = os.path.split(filename)
@@ -158,22 +95,22 @@ def search_path_for(filename):
             return realpath(fullname)
     return None
 
-def find_package_for(filename):
-    """Find the package(s) containing this file."""
-    packages = {}
-    newfilename = search_path_for(filename)
-    if newfilename:
-        filename = newfilename
-        
-    pipe = os.popen("dpkg --search '%s'" % filename)
+def query_dpkg_for(filename):
+    try:
+        x = os.getcwd()
+    except OSError:
+        os.chdir('/')
+    pipe = os.popen("COLUMNS=79 dpkg --search '%s' 2>/dev/null" % filename)
     matcher = re.compile('diversion by')
+    packages = {}
 
     for line in pipe.readlines():
         line = string.strip(line)
         # Ignore diversions
         if matcher.match(line): continue
 
-        (package, path) = string.split(line, ':')
+        (package, path) = string.split(line, ':', 1)
+        path = string.strip(path)
         packlist = string.split(package, ', ')
         for package in packlist:
             if packages.has_key(package):
@@ -183,59 +120,128 @@ def find_package_for(filename):
     pipe.close()
     return filename, packages
 
+def find_package_for(filename):
+    """Find the package(s) containing this file."""
+    packages = {}
+    if filename[0] == '/':
+        file, pkglist = query_dpkg_for(filename)
+        if pkglist: return file, pkglist
+        
+    newfilename = search_path_for(filename)
+    return query_dpkg_for(newfilename or filename)
+
 def quote_if_needed(realname):
     acceptable = string.letters+string.digits+' '
     quote = 0
     for char in realname:
         if char not in acceptable:
             quote = 1
-    
+
     if quote:
-        return '"%s"' % realname
-    else:
-        return realname
+        return '"'+realname+'"'
+    return realname
+
+def find_rewritten(username):
+    for filename in ['/etc/email-addresses']:
+        if os.path.exists(filename):
+             fp = open(filename)
+             for line in fp.readlines():
+                 line = string.strip(line)
+                 line = string.split(line, '#')[0]
+                 if not line:
+                     continue
+                 try:
+                     name, alias = string.split(line, ':')
+                     if string.strip(name) == username:
+                         return string.strip(alias)
+                 except ValueError:
+                     print 'Invalid entry in %s' % filename
+                     return None
         
+# Swiped directly from Python 2.2 CVS. (C) 2001 Python Software Foundation
+# See /usr/share/doc/python2.{1,2}/copyright for license.
+def getfqdn(name=''):
+    """Get fully qualified domain name from name.
+
+    An empty argument is interpreted as meaning the local host.
+
+    First the hostname returned by gethostbyaddr() is checked, then
+    possibly existing aliases. In case no FQDN is available, hostname
+    is returned.
+    """
+    name = string.strip(name)
+    if not name or name == '0.0.0.0':
+        name = socket.gethostname()
+    try:
+        hostname, aliases, ipaddrs = socket.gethostbyaddr(name)
+    except socket.error:
+        pass
+    else:
+        aliases.insert(0, hostname)
+        for name in aliases:
+            if '.' in name:
+                break
+        else:
+            name = hostname
+    return name
+
 def get_user_id(email='', realname=''):
     uid = os.getuid()
     info = pwd.getpwuid(uid)
-    email = os.environ.get('DEBEMAIL') or os.environ.get('EMAIL') or email
-    if not email:
-        if not os.path.exists('/etc/mailname'):
-            sys.stderr.write('Please set the environment variable DEBEMAIL '
-                             'or EMAIL and try again.\n')
-            sys.exit(1)
+    email = (os.environ.get('REPORTBUGEMAIL', email) or
+             os.environ.get('DEBEMAIL') or os.environ.get('EMAIL'))
+    
+    email = email or find_rewritten(info[0]) or info[0]
 
-        fp = open('/etc/mailname')
-        domainname = string.strip(fp.readline())
-        fp.close()
+    if '@' not in email:
+        if os.path.exists('/etc/mailname'):
+            domainname = string.strip(open('/etc/mailname').readline())
+        else:
+            domainname = getfqdn()
 
-        email = info[0]+'@'+domainname
+        email = email+'@'+domainname
 
     # Handle EMAIL if it's formatted as 'Bob <bob@host>'.
     if '<' in email:
-        return email
+        # Handle unquoted realname parts
+        m = re.match(r'([^"].+[^"])*\s+<(.*)>', email)
+        if not m: return email
+        realname, email = m.groups()
         
     realname = realname or string.split(info[4], ',')[0]
-    realname = os.environ.get('DEBNAME') or os.environ.get('NAME') or \
-               os.environ.get('USERNAME') or realname
-
+    realname = (os.environ.get('DEBFULLNAME') or os.environ.get('DEBNAME') or
+                os.environ.get('NAME') or realname)
     realname = quote_if_needed(realname)
     return '%s <%s>' % (realname, email)
     
 def get_package_status(package):
     versionre = re.compile('Version: ')
     packagere = re.compile('Package: ')
+    priorityre = re.compile('Priority: ')
     dependsre = re.compile('(Pre-)?Depends: ')
-    conffilesre = re.compile('Conffiles:')
-    maintre = re.compile('Maintainer:')
+    conffilesre = re.compile('Conffiles: ')
+    maintre = re.compile('Maintainer: ')
+    statusre = re.compile('Status: ')
+    originre = re.compile('Origin: ')
+    bugsre = re.compile('Bugs: ')
+    descre = re.compile('Description: ')
+    fullre = re.compile(' ')
+    srcre = re.compile('Source: ')
 
-    pkgversion = pkgavail = depends = maintainer = None
+    pkgversion = pkgavail = depends = maintainer = status = origin = None
+    bugs = vendor = priority = desc = src_name = None
     conffiles = []
-    
+    fulldesc = []
     confmode = 0
-    pipe = os.popen("dpkg --status '%s'" % package)
+    
+    try:
+        x = os.getcwd()
+    except OSError:
+        os.chdir('/')
+    pipe = os.popen("COLUMNS=79 dpkg --status '%s' 2>/dev/null" % package)
+    
     for line in pipe.readlines():
-        line = string.strip(line)
+        line = string.rstrip(line)
         if not line: continue
 
         if confmode:
@@ -245,11 +251,21 @@ def get_package_status(package):
                 conffiles.append(string.split(line))
 
         if versionre.match(line):
-            (crud, pkgversion) = string.split(line, ": ")
+            (crud, pkgversion) = string.split(line, ": ", 1)
+        elif statusre.match(line):
+            (crud, status) = string.split(line, ": ", 1)
+        elif priorityre.match(line):
+            (crud, priority) = string.split(line, ": ", 1)
         elif packagere.match(line):
-            (crud, pkgavail) = string.split(line, ": ")
+            (crud, pkgavail) = string.split(line, ": ", 1)
+        elif originre.match(line):
+            (crud, origin) = string.split(line, ": ", 1)
+        elif bugsre.match(line):
+            (crud, bugs) = string.split(line, ": ", 1)
+        elif descre.match(line):
+            (crud, desc) = string.split(line, ": ", 1)
         elif dependsre.match(line):
-            (crud, thisdepends) = string.split(line, ": ")
+            (crud, thisdepends) = string.split(line, ": ", 1)
             if depends:
                 depends = depends+', '+thisdepends
             else:
@@ -257,18 +273,84 @@ def get_package_status(package):
         elif conffilesre.match(line):
             confmode = 1
         elif maintre.match(line):
-            crud, maintainer = string.split(line, ": ")
+            crud, maintainer = string.split(line, ": ", 1)
+        elif srcre.match(line):
+            crud, src_name = string.split(line, ": ", 1)
+        elif desc and line[0]==' ':
+            fulldesc.append(line)
 
     pipe.close()
 
-    return (pkgversion, pkgavail, depends, conffiles, maintainer)
+    installed = 0
+    if status:
+        state = string.split(status)[2]
+        installed = (state not in ('config-files', 'not-installed'))
+
+    reportinfo = None
+    if bugs:
+        reportinfo = debianbts.parse_bts_url(bugs)
+    elif origin:
+        if debianbts.SYSTEMS.has_key(origin):
+            vendor = debianbts.SYSTEMS[origin]['name']
+            reportinfo = (debianbts.SYSTEMS[origin]['type'],
+                          debianbts.SYSTEMS[origin]['btsroot'])
+            
+        else:
+            vendor = string.capitalize(origin)
+    else:
+        vendor = ''
+
+    return (pkgversion, pkgavail, depends, conffiles, maintainer, installed,
+            origin, vendor, reportinfo, priority, desc, src_name,
+            string.join(fulldesc, os.linesep))
+
+dbase = None
+def get_dpkg_database():
+    global dbase
+
+    if not dbase:
+        dbase = open('/var/lib/dpkg/status').read()
+
+    return dbase
+
+avail = None
+def get_avail_database():
+    global avail
+
+    if not avail:
+        avail = open('/var/lib/dpkg/available').read()
+
+    return avail
+
+def get_source_package(package):
+    """Return any binary packages provided by a source package."""
+    dbase = get_avail_database()
+    packages = []
+    packob = re.compile(r'^Package: (?P<pkg>.*)$', re.MULTILINE)
+    descob = re.compile(r'^Description: (?P<desc>.*)$', re.MULTILINE)
+    searchob = re.compile(r'^(Package|Source): '+re.escape(package)+r'$',
+                          re.MULTILINE)
+    
+    packinfo = string.split(dbase, '\n\n')
+    for p in packinfo:
+        match = searchob.search(p)
+        if match:
+            packname = packdesc = ''
+            namematch, descmatch = packob.search(p), descob.search(p)
+
+            if namematch:
+                packname = namematch.group('pkg')
+            if descmatch:
+                packdesc = descmatch.group('desc')
+            
+            if packname:
+                packages.append( (packname, packdesc) )
+
+    packages.sort()
+    return packages
 
 def get_package_info(packages):
-    # Package: package | Provides: package
-    fp = open('/var/lib/dpkg/status')
-    dbase = fp.read()
-    fp.close()
-
+    dbase = get_dpkg_database()
     pkgname = r'(?:[\S]+(?:$|,\s+))'
 
     groupfor = {}
@@ -322,6 +404,15 @@ def get_package_info(packages):
 
     return ret
 
+def packages_providing(package):
+    aret = get_package_info([('', package)])
+    ret = []
+    for pkg in aret:
+        ret.append( (pkg[0], pkg[3]) )
+        if not pkg[2]: return []
+    
+    return ret
+
 def get_dependency_info(package, depends):
     depends = string.split(depends, ', ')
     dependencies = []
@@ -347,7 +438,7 @@ def get_dependency_info(package, depends):
         
         packstuff = '%-*.*s %s' % (39-maxlen, 39-maxlen, pack, vers)
                 
-        info = '%-3.3s %-40.40s %-34.34s\n' % (status, packstuff, desc)
+        info = '%-3.3s %-40.40s %-.34s\n' % (status, packstuff, desc)
         depinfo = depinfo + info
     
     return depinfo
@@ -356,7 +447,12 @@ def old_get_dependency_info(package, depends):
     dependencies = string.split(depends, ', ')
     dependencies = map(lambda x: string.split(x, ' ')[0], dependencies)
 
-    pipe = os.popen("dpkg -l "+string.join(dependencies)+" | uniq")
+    try:
+        x = os.getcwd()
+    except OSError:
+        os.chdir('/')
+    pipe = os.popen("COLUMNS=79 dpkg -l "+string.join(dependencies)+" | uniq")
+
     pkgre = re.compile('[uirp]')
     depinfo = "\nVersions of packages %s depends on:\n" % package
     for line in pipe.readlines():
@@ -393,16 +489,19 @@ def get_changed_config_files(conffiles, nocompress=0):
 
     return confinfo, changed
 
-def generate_blank_report(package, pkgversion, severity, depinfo, confinfo,
-                          foundfile='', incfiles='', system='debian'):
-    if SYSTEMS[system]['query-dpkg']:
+def generate_blank_report(package, pkgversion, severity, justification,
+                          depinfo, confinfo, foundfile='', incfiles='',
+                          system='debian', exinfo=0, type=None, klass='',
+                          subject='', tags='', body=''):
+    if debianbts.SYSTEMS[system]['query-dpkg']:
         debvers = ''
         if os.path.exists('/etc/debian_version'):
             file = open('/etc/debian_version')
             debvers = string.strip(file.readline())
             file.close()
 
-        debarch = commands.getoutput('dpkg --print-installation-architecture')
+        debarch = commands.getoutput('COLUMNS=79 dpkg '
+                                     '--print-installation-architecture')
         debinfo = "Debian Release: %s\nArchitecture: %s\n" % (debvers, debarch)
     else:
         debinfo = ''
@@ -412,12 +511,29 @@ def generate_blank_report(package, pkgversion, severity, depinfo, confinfo,
     else:
         fileinfo = ''
 
-    if SYSTEMS[system].has_key('namefmt'):
-        package = SYSTEMS[system]['namefmt'] % package
+    locinfo = []
+    langsetting = os.environ.get('LANG', 'C')
+    allsetting = os.environ.get('LC_ALL', '')
+    for setting in ('LANG', 'LC_CTYPE'):
+        if setting == 'LANG':
+            env = langsetting
+        else:
+            env = allsetting or os.environ.get(setting, langsetting)
+        locinfo.append('%s=%s' % (setting, env))
+    locinfo = string.join(locinfo, ', ')
+
+    if debianbts.SYSTEMS[system].has_key('namefmt'):
+        package = debianbts.SYSTEMS[system]['namefmt'] % package
 
     if severity:
         severity = 'Severity: %s\n' % severity
 
+    if justification:
+        severity = severity + 'Justification: %s\n' % justification
+
+    if tags:
+        fileinfo = fileinfo + 'Tags: %s\n' % tags
+
     specials = ''
 ##    specialpath = '/usr/share/doc/%s/Debian.bugtemplate' % package
 ##    if os.path.exists(specialpath) and os.path.isfile(specialpath):
@@ -427,15 +543,44 @@ def generate_blank_report(package, pkgversion, severity, depinfo, confinfo,
 
 ##        specials = '-- Special Instructions\n' + specials + '\n'
 
-    report = """Package: %s
-Version: %s
-%s%s
-%s
+    
+    report = "\n"
+    if not exinfo:
+        if type == 'gnats':
+            report = ">Synopsis: %s\n>Confidential: no\n" % subject
+            if package == 'debian-general':
+                report = report + ">Category: %s\n" % package
+            else:
+                report = report + ">Category: debian-packages\n"\
+                         ">Release: %s_%s\n" % (package, pkgversion)
+
+            if severity:
+                report = report + ">"+severity
+
+            if klass:
+                report = report + ">Class: %s\n" % klass
+            report = report + (
+                ">Description:\n\n"
+                "  <describe the bug here; use as many lines as you need>\n\n"
+                ">How-To-Repeat:\n\n"
+                "  <show how the bug is triggered>\n\n"
+                ">Fix:\n\n"
+                "  <if you have a patch or solution, put it here>\n\n"
+                ">Environment:\n")
+        else:
+            report = "Package: %s\nVersion: %s\n%s%s\n" % (package, pkgversion,
+                                                           severity, fileinfo)
+    else:
+        report = "Followup-For: Bug #%d\nPackage: %s\nVersion: %s\n" % (
+            exinfo, package, pkgversion)
+
+    report = report + body + """%s
 
 %s-- System Information
 %sKernel: %s
-%s%s""" % (package, pkgversion, severity, fileinfo, incfiles, specials,
-           debinfo, string.join(os.uname()), depinfo, confinfo)
+Locale: %s
+%s%s""" % (incfiles, specials, debinfo, string.join(os.uname()), locinfo,
+           depinfo, confinfo)
 
     return report
 
@@ -449,6 +594,12 @@ class our_lex(shlex.shlex):
 
 FILES = ('/etc/reportbug.conf', '~/.reportbugrc')
 
+CONFIG_ARGS = ('sendto', 'severity', 'mua', 'mta', 'email', 'realname', 'bts',
+               'replyto', 'http_proxy', 'smtphost', 'editor', 'mua', 'mta',
+               'justification', 'sign', 'nocc', 'nocompress', 'dontquery',
+               'noconf', 'use_ldap', 'mirrors', 'headers', 'interface',
+               'template')
+
 def parse_config_files():
     args = {}
     for file in FILES:
@@ -468,16 +619,14 @@ def parse_config_files():
                     args['sendto'] = token
                 elif token == 'severity':
                     token = string.lower(lex.get_token())
-                    if token in SEVERITIES.values():
+                    if token in debianbts.SEVERITIES.keys():
                         args['severity'] = token
-                elif token == 'mua':
-                    args['mua'] = lex.get_token()
                 elif token == 'mutt':
                     args['mua'] = 'mutt -H'
                 elif token == 'af':
                     args['mua'] = 'af -EH <'
                 elif token in ('mh', 'nmh'):
-                    args['mua'] = '/usr/bin/mh/comp -draftmessage'
+                    args['mua'] = '/usr/bin/mh/comp -use -file'
                 elif token == 'header':
                     args['headers'] = args.get('headers', []) + \
                                       [lex.get_token()]
@@ -489,28 +638,24 @@ def parse_config_files():
                     args['nocompress'] = 1
                 elif token == 'compress':
                     args['nocompress'] = 0
-                elif token == 'no-bts-query':
+                elif token == 'no-query-bts':
                     args['dontquery'] = 1
-                elif token == 'bts-query':
+                elif token == 'query-bts':
                     args['dontquery'] = 0
                 elif token == 'config-files':
                     args['noconf'] = 0
                 elif token == 'no-config-files':
                     args['noconf'] = 1
                 elif token == 'ldap':
-                    args['ldap'] = 1
+                    args['use_ldap'] = 1
                 elif token == 'no-ldap':
-                    args['ldap'] = 0
-                elif token == 'email':
-                    args['email'] = lex.get_token()
-                elif token == 'realname':
-                    args['realname'] = lex.get_token()
-                elif token == 'replyto':
-                    args['replyto'] = lex.get_token()
-                elif token == 'http_proxy':
-                    args['http_proxy'] = lex.get_token()
-                elif token == 'smtphost':
-                    args['smtphost'] = lex.get_token()
+                    args['use_ldap'] = 0
+                elif token in ('printonly', 'template'):
+                    args[token] = 1
+                elif token in ('email', 'realname', 'replyto', 'http_proxy',
+                               'smtphost', 'editor', 'mua', 'mta',
+                               'justification'):
+                    args[token] = lex.get_token()
                 elif token == 'sign':
                     token = string.lower(lex.get_token())
                     if token in ('pgp', 'gpg'):
@@ -519,16 +664,39 @@ def parse_config_files():
                         args['sign'] = 'gpg'
                     elif token == 'none':
                         args['sign'] = ''
+                elif token == 'ui':
+                    token = string.lower(lex.get_token())
+                    if token in VALID_UIS:
+                        args['interface'] = token
                 elif token == 'bts':
                     token = string.lower(lex.get_token())
-                    if token in SYSTEMS.keys():
+                    if token in debianbts.SYSTEMS.keys():
                         args['bts'] = token
                 elif token == 'mirror':
                     args['mirrors'] = args.get('mirrors', []) + \
                                       [lex.get_token()]
+                elif token in ('reportbug_version', 'mode'):
+                    token = lex.get_token()
                 else:
                     sys.stderr.write('Unrecognized token: '+token+'\n')
 
                 token = lex.get_token()
 
     return args
+
+def parse_bug_control_file(filename):
+    submitas = submitto = None
+    fh = open(filename)
+    for line in fh.readlines():
+        line = string.strip(line)
+        parts = string.split(line, ': ')
+        if len(parts) != 2:
+            continue
+
+        header, data = parts
+        if string.lower(header) == 'submit-as':
+            submitas = data
+        elif string.lower(header) == 'send-to':
+            submitto = data
+
+    return submitas, submitto
